@@ -11,8 +11,12 @@
 			<string>SCRIPTDIR</string>
 		</array>
 #ifdef TIGER_LAUNCHD
+	<key>RunAtLoad</key>
+		<true/>
 	<key>KeepAlive</key>
 		<true/>
+	<key>ServiceIPC</key>
+		<true/>
 #else
 	<key>MachServices</key>
 		<dict>
